# build_mcu_fpga_batch.tcl
#
# cmsdk_mcu sample design 
# A Vivado script that demonstrates a very simple RTL-to-bitstream non-project batch flow
#
# NOTE:  typical usage would be "vivado -mode tcl -source build_mcu_fpga_batch.tcl" 
#
# STEP#0: define output directory area.
#

##if {$argc < 1} {
#puts  "target_fpga arg must be \[ac701 \| arm_mps3 \| pynz_z2 \| zcu104\]"
#}
#set target [lindex $argv 0]
#puts "target requested : $target"
#set target_dir target_fpga
#append target_dir $target
#puts "target directory : $target_dir"

set outputDir ./vivado/built_mcu_fpga
file mkdir $outputDir
#
# STEP#1: setup design sources and constraints
#

# local search path for configurations
set search_path ../verilog

set cortexm0_vlog    ../../../../../../arm-AAA-ip/Cortex-M0/AT510-BU-00000-r0p0-03rel3/logical
source scripts/rtl_source_cm0.tcl

set search_path [ concat $search_path $cortexm0_vlog/cortexm0_integration/verilog ]
read_verilog  [ glob $cortexm0_vlog/cortexm0_integration/verilog/*.v ]
read_verilog  [ glob $cortexm0_vlog/models/cells/*.v ]

# Arm unmodified CMSDK RTL
set cmsdk_vlog    ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0
source scripts/rtl_source_cmsdk.tcl

set search_path [ concat $search_path $cmsdk_vlog/logical/models/memories ]
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_memory_models_defs.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_rom.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_fpga_rom.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_ram.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_fpga_sram.v

# ADP, FT1248 and streamio IP
source scripts/rtl_source_soclabs_ip.tcl

# FPGA-specific pads
source scripts/rtl_source_fpga_ip.tcl

# soclabs modified mcu system 
set soc_vlog ../verilog
read_verilog  $soc_vlog/cmsdk_mcu_defs.v
read_verilog  $soc_vlog/ahb_bootrom.v
read_verilog  $soc_vlog/bootrom.v
read_verilog  $soc_vlog/cmsdk_ahb_cs_rom_table.v
read_verilog  $soc_vlog/cmsdk_apb_usrt.v
read_verilog  $soc_vlog/cmsdk_mcu_addr_decode.v
read_verilog  $soc_vlog/cmsdk_mcu_clkctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_pin_mux.v
read_verilog  $soc_vlog/cmsdk_mcu_stclkctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_sysctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_system.v
read_verilog  $soc_vlog/cmsdk_mcu_chip.v

# FPGA specific timing constraints
read_xdc target_fpga/fpga_timing.xdc

# FPGA board specific pin constraints
read_xdc target_fpga/fpga_pinmap.xdc

#
# STEP#2: run synthesis, report utilization and timing estimates, write checkpoint design
#
source target_fpga/fpga_synth.tcl

write_checkpoint -force $outputDir/post_synth
report_timing_summary -file $outputDir/post_synth_timing_summary.rpt
report_power -file $outputDir/post_synth_power.rpt
#
# STEP#3: run placement and logic optimzation, report utilization and timing estimates, write checkpoint design
#
opt_design
place_design
phys_opt_design
write_checkpoint -force $outputDir/post_place
report_timing_summary -file $outputDir/post_place_timing_summary.rpt
#
# STEP#4: run router, report actual utilization and timing, write checkpoint design, run drc, write verilog and xdc out
#
route_design
write_checkpoint -force $outputDir/post_route
report_timing_summary -file $outputDir/post_route_timing_summary.rpt
report_timing -sort_by group -max_paths 100 -path_type summary -file $outputDir/post_route_timing.rpt
report_clock_utilization -file $outputDir/clock_util.rpt
report_utilization -file $outputDir/post_route_util.rpt
report_power -file $outputDir/post_route_power.rpt
report_drc -file $outputDir/post_imp_drc.rpt
write_verilog -force $outputDir/cmsdk_mcu_impl_netlist.v
write_xdc -no_fixed_only -force $outputDir/cmsdk_mcu_impl.xdc
#
# STEP#5: generate a bitstream
# 
write_bitstream -force $outputDir/cmsdk_mcu.bit
